# Copyright (c) 2013, University of Bristol and University of Southampton.

from EStat.Templating import Template

class Template1LevelMVMixedAML(Template):
    'Fits 1-level multivariate mixed response models (Normal, binomial, ordered, or unordered) in aML only.'

    __version__ = '1.0.0'

    tags = [ 'Model', '1-Level', 'Normal', 'Binomial', 'Ordered multinomial', 'Unordered multinomial', 'Probit', 'Multivariate response']
    engines = ['aML']

    inputs = '''
y = DataMatrix('Responses: ')
n = 0
lenbeta = 0
for i in range(0, len(y)):
    context['bin'+str(i)] = Text('Response Type for response ' + y[i] + ': ', ['Normal', 'Binomial', 'Ordered', 'Unordered'])
    if(context['bin'+str(i)] == 'Unordered'):
        context['ncats'+y[i]] = Integer('Number of categories for ' + y[i] + ': ')
        context['base' + y[i]] = Integer('Base category: ')
        for j in range(0, context['ncats'+y[i]]-1):
            context['x'+str(i)+'_'+str(j)] = DataMatrix('Explanatory variables for category ' + str(j+1) + ': ', allow_cat = True)
            lenbeta += len(context['x'+str(i)+'_'+str(j)])
        n += context['ncats'+y[i]] - 1
    else:
        context['x'+str(i)] = DataMatrix('Explanatory variables for response ' + y[i] + ': ', allow_cat = True)
        lenbeta += len(context['x'+str(i)])
        n += 1

#n = len(y)
omega_e = ParamMatrix()
omega_e.size = n

beta = ParamVector()
beta.ncols = lenbeta
'''

    amlscript = '''
<%
import numpy 
maxcat = {}
mincat = {}
numcat = {}
cats = {}

n = 0
for i in range(0, len(y)):
    if(context['bin'+str(i)] == 'Unordered'):
        n += context['ncats'+y[i]] - 1
    else:
        n += 1
%>

option iterations=10000;

% for i in range(0, len(y)):
% if context['bin' + str(i)] == 'Ordered':
<%
mincat[i] = int(numpy.min(dsets['datafile'].variables[y[i]]['data']))
maxcat[i] = int(numpy.max(dsets['datafile'].variables[y[i]]['data']))
numcat[i] = maxcat[i] - mincat[i]
%>
define vector Taus${i}; dim=${numcat[i]};
% endif
% if context['bin' + str(i)] == 'Unordered':
<%
cats[i] = list(set(dsets['datafile'].variables[y[i]]['data']))
cats[i].remove(int(context['base' + y[i]]))
print str(cats[i])
%>
% for j in range(0, len(cats[i])):
define regressor set Cat${int(cats[i][j])};
var = \\
% for p in context['x' + str(i)+'_'+str(j)]:
${p} \\
% endfor
;
% endfor
% else:
define regressor set BetaX${i};
var = \\
% for p in context['x' + str(i)]:
${p} \\
% endfor
;
% endif
% endfor


define normal distribution; dim=${n};
% for i in range(0, n):
    name=e${i};
% endfor

<% n = 0 %>\\
% for i in range(0, len(y)):
% if context['bin' + str(i)] == 'Normal':
continuous model;
    outcome = ${y[i]};
    model = regset BetaX${i} +
        res(draw=_id, ref=e${n});
<% n += 1 %>\\
% endif

% if context['bin' + str(i)] == 'Binomial':
probit model;
    outcome = ${y[i]};
    model = regset BetaX${i} +
        res(draw=_id, ref=e${n});
<% n += 1 %>\\
% endif

% if context['bin' + str(i)] == 'Ordered':
ordered probit model;
    outcome = ${y[i]}-${mincat[i]} ${y[i]}+${1-mincat[i]};
    thresholds = Taus${i};
    model = regset BetaX${i} +
        res(draw=_id, ref=e${n});
<% n += 1 %>\\
% endif


% if context['bin' + str(i)] == 'Unordered':
multinomial probit model;
    outcome = ${y[i]};
% for j in range(0, len(cats[i])):
    model ${int(cats[i][j])} = regset Cat${int(cats[i][j])} +
        res(draw=_id, ref=e${n});
<% n += 1 %>\\
% endfor
% endif

% endfor

starting values;
% for i in range(0, len(y)):
% if context['bin' + str(i)] == 'Ordered':
% for j in range(0, numcat[i]):
Tau${i}_${j}	T	${j - (numcat[i]/2.0)}
% endfor
% endif
% if context['bin' + str(i)] == 'Unordered':
% for k in range(0, len(cats[i])):
% for j in range(0, len(context['x' + str(i) + '_' + str(j)])):
bet${j}_${i}_${int(cats[i][k])}	T	0
% endfor
% endfor
% else:
% for j in range(0, len(context['x' + str(i)])):
beta${j}_${i}	T	0
% endfor
% endif
% endfor
<% betan = 0 %>\\
% for i in range(0, len(y)):
% if context['bin'+str(i)] == 'Normal':
Sig${betan}	T	1
<% betan += 1%>\\
% elif context['bin' + str(i)] == 'Unordered':
% for k in range(0, len(cats[i])):
Sig${betan}	F	1
<% betan += 1%>\\
% endfor
% else:
Sig${betan}	F	1
<% betan += 1%>\\
% endif
% endfor
% for i in range(0, n):
% for j in range(0, i+1):
% if i != j:
Rho${i}_${j}	T	0
% endif
% endfor
% endfor
;
'''
