# Copyright (c) 2013, University of Bristol and University of Southampton.

from EStat.Templating import Template

class Template1LevelMVNormalAML(Template):
    'Fits 1-level multivariate Normal response models, using aML only.'

    __version__ = '1.0.0'

    tags = [ 'Model', '1-Level', 'Normal', 'Multivariate response' ]
    engines = ['aML']

    inputs = '''
y = DataMatrix('Responses: ')
lenbeta = 0
for i in range(0, len(y)):
    context['x'+str(i)] = DataMatrix('Explanatory variables for response ' + y[i] + ': ', allow_cat = True)
    lenbeta += len(context['x'+str(i)])

n = len(y)
omega_e = ParamMatrix()
omega_e.size = n

beta = ParamVector()
beta.ncols = lenbeta
'''
    
    amlscript = '''
<% n = len(y) %>

option iterations=500;

% for i in range(0, n):
define regressor set BetaX${i};
var = \\
% for p in context['x' + str(i)]:
${p} \\
% endfor
;
% endfor

define normal distribution; dim=${n};
% for i in range(0, n):
    name=e${i};
% endfor

% for i in range(0, n):
continuous model;
    outcome = ${y[i]};
    model = regset BetaX${i} +
        res(draw=_id, ref=e${i});
% endfor

starting values;
% for i in range(0, n):
% for j in range(0, len(context['x' + str(i)])):
beta${j}_${i}	T	0
% endfor
% endfor
% for i in range(0, n):
Sig${i}	T	1
% endfor
% for i in range(0, n):
% for j in range(0, i+1):
% if i != j:
Rho${i}_${j}	T	0
% endif
% endfor
% endfor
;
'''
