# Copyright (c) 2013, University of Bristol and University of Southampton.

from EStat.Templating import Template

class Template1LevelModAML(Template):
    'Fits 1-level models for Normal, Poisson, binary, binomial and negative binomial responses, using aML only.'

    __version__ = '1.0.0'

    tags = [ 'Model', '1-Level', 'Normal', 'Binomial', 'Negative binomial', 'Poisson', 'Probit', 'Logit' ]
    engines = ['aML']

    inputs = '''
y = DataVector('Response: ')
D = Text('Specify distribution: ', ['Normal', 'Binomial', 'Binary', 'Poisson', 'Negative Binomial'])
if D == 'Binomial':
    n = DataVector('Denominator: ')
    link = Text('Specify link function: ', ['logit', 'probit'])
if D == 'Binary':
    link = Text('Specify link function: ', ['logit', 'probit'])
if D == 'Poisson':
    link = Text(value = 'ln')
    offset = Boolean('Is there an offset: ')
    if offset:
        n = DataVector('Offset: ')
if D == 'Normal':
    sigma = ParamScalar()
if D == 'Negative Binomial':
    offset = Boolean('Is there an offset: ')
    if offset:
        n = DataVector('Offset: ')
x = DataMatrix('Explanatory variables: ', allow_cat = True)
beta = ParamVector(parents=[x], as_scalar=True)
'''
    
    amlscript = '''
define regressor set BetaX;
var = \\
% for p in x:
${p} \\
% endfor
;

% if D == 'Normal':
define normal distribution; dim=1;
    name=e;

continuous model;
    outcome = ${y};
    model = regset BetaX +
        res(draw=_iid, ref=e);
% endif

% if D == 'Binomial':
binomial model;
    outcome = ${y};
    exposure = ${n};
% if link == 'logit':
    probability = logistic(regset BetaX);
% endif
% if link == 'probit':
    probability = normprob(regset BetaX);
% endif
% endif
% if D == 'Binary':
% if link == 'logit':
logit model;
% endif
% if link == 'probit':
probit model;
% endif
  outcome=use;
  model = regset BetaX;
% endif
% if D == 'Poisson':
poisson model;
    outcome = ${y};
% if offset:
    incidence = exp(${n} + regset BetaX);
% else:
    incidence = exp(regset BetaX);
% endif
% endif
% if D == 'Negative Binomial':
define parameter Alpha; range=(0,Inf);

negative binomial model;
    outcome = ${y};
    dispersion = par Alpha;
% if offset:
    incidence = exp(${n} + regset BetaX);
% else:
    incidence = exp(regset BetaX);
% endif
% endif

starting values;
% for i in range(0, len(x)):
beta${i}	T	0
% endfor
% if D == 'Normal':
sigma	T	1
% endif
% if D == 'Negative Binomial':
Alpha	F	1
% endif
;
'''

    latex = r'''
\begin{aligned}
 \mbox{${y}}_i & \sim \mbox{N}(\mu_i, \sigma^2) \\ 
\mu_i & =
  ${mmulttex(x, r'\beta', 'i')} \\ 
\end{aligned} 
'''
