# Copyright (c) 2013, University of Bristol and University of Southampton.

from EStat.Templating import Template

class Template2LevelAML(Template):
    'A model template for fitting 2 level Normal multiple regression models, using aML only.'

    __version__ = '1.0.0'

    tags = [ 'Model', '2-Level', 'Normal' ]
    engines = ['aML']

    inputs = '''
ID = IDVector('Identifier variable')
y = DataVector('Response: ')
sigma = ParamScalar()
sigmau = ParamScalar()
x = DataMatrix('Explanatory variables: ', allow_cat = True)
beta = ParamVector(parents=[x], as_scalar=True)
'''
    
    amlscript = '''

option maximum number of residual draws = 200;

define regressor set BetaX;
var = \\
% for p in x:
${p} \\
% endfor
;

define normal distribution; dim=1;
    name=u;

define normal distribution; dim=1;
    name=e;

continuous model;
    outcome = ${y};
    model = regset BetaX +
        res(draw=1, ref=u) + res(draw=_iid, ref=e);

starting values;
% for i in range(0, len(x)):
beta${i}	T	0
% endfor
sigmau	T	1
sigma	T	1
;
'''

    latex = r'''
\begin{aligned}
 \mbox{${y}}_i & \sim \mbox{N}(\mu_i, \sigma^2) \\ 
\mu_i & =
  ${mmulttex(x, r'\beta', 'i')} \\ 
\end{aligned} 
'''
