# Copyright (c) 2013, University of Bristol and University of Southampton.

from EStat.Templating import Template

class Template2LevelNormRSAML(Template):
    'Fits 2-level models for Normal responses, allowing for random slopes - aML only.'

    __version__ = '1.0.0'

    tags = [ 'Model', '2-Level', 'Normal', 'Random slopes']
    engines = ['aML']

    inputs = '''
ID = IDVector('Identifier variable')
y = DataVector('Response: ')
sigma = ParamScalar()
sigmau = ParamScalar()
x = DataMatrix('Explanatory variables: ', allow_cat = True)
beta = ParamVector(parents=[x], as_scalar=True)
x2 = DataMatrix('Explanatory variables random at level 2: ', allow_cat = True)
for var in range(0, len(x2.name)):
    context['sigu' + str(var)] = ParamScalar()
'''
   
    amlscript = '''

option maximum number of residual draws = 200;
OPTION MAXIMUM MODEL SPACE = 15000;

define regressor set BetaX;
var = \\
% for p in x:
${p} \\
% endfor
;

define normal distribution; dim=${len(x2)};
% for i in range(0, len(x2)):
    name=u${i};
% endfor

define normal distribution; dim=1;
    name=e;

continuous model;
    outcome = ${y};
    model = regset BetaX\\
% for i in range(0, len(x2)):
 + ${x2[i]}*res(draw=1, ref=u${i})
% endfor
 + res(draw=_iid, ref=e);

starting values;
% for i in range(0, len(x)):
beta${i}	T	0
% endfor
% for i in range(0, len(x2)):
SigmaU${i}	T	1
% endfor
% for i in range(0, len(x2)):
% for j in range(0, i+1):
% if i != j:
RhoU${j}U${i}	T	0
% endif
% endfor
% endfor
sigma	T	1
;
'''

    latex = r'''
\begin{aligned}
 \mbox{${y}}_i & \sim \mbox{N}(\mu_i, \sigma^2) \\ 
\mu_i & =
  ${mmulttex(x, r'\beta', 'i')} \\ 
%for i in range(0, len(x)):
\beta_${i} & \propto 1 \\ 
%endfor
\tau & \sim \Gamma (0.001,0.001) \\ 
\sigma^2 & = 1 / \tau
\end{aligned} 
'''
