# Copyright (c) 2013, University of Bristol and University of Southampton.

from EStat.Templating import Template

class NLevelNormAML(Template):
    'Fits Normal response models, in aML only, allowing one or more higher classifications (nested or crossed).'

    __version__ = '1.0.0'

    tags = [ 'Model', 'N-Level', 'Normal' ]
    engines = ['aML']

    inputs = '''
NumLevs = Integer('Number of Classifications: ')
for i in range(0, int(NumLevs)):
    selstr = 'Classification ' + str(i + 1) + ': '
    context['C' + str(i + 1)] = IDVector(selstr)
    context['sigmau' + str(i + 1)] = ParamScalar()

ID = context['C'+str(int(NumLevs))]
y = DataVector('Response: ')
sigma = ParamScalar()
x = DataMatrix('Explanatory variables: ', allow_cat = True)
beta = ParamVector(parents=[x], as_scalar=True)
'''
    
    amlscript = '''

option maximum number of residual draws = 200;

define regressor set BetaX;
var = \\
% for p in x:
${p} \\
% endfor
;

% for i in range(0, int(NumLevs)):
define normal distribution; dim=1;
    name=u${i};
% endfor

define normal distribution; dim=1;
    name=e;

continuous model;
    outcome = ${y};
    model = regset BetaX 
% for i in range(0, int(NumLevs)):
        + res(draw=${context['C' + str(i + 1)]}, ref=u${i})
% endfor
        + res(draw=_iid, ref=e);

starting values;
% for i in range(0, len(x)):
beta${i}	T	0
% endfor
% for i in range(0, int(NumLevs)):
sigmau${i}	T	1
% endfor
sigma	T	1
;
'''

    latex = r'''
\begin{aligned}
 \mbox{${y}}_i & \sim \mbox{N}(\mu_i, \sigma^2) \\ 
\mu_i & =
  ${mmulttex(x, r'\beta', 'i')} \\ 
%for i in range(0, len(x)):
\beta_${i} & \propto 1 \\ 
%endfor
\tau & \sim \Gamma (0.001,0.001) \\ 
\sigma^2 & = 1 / \tau
\end{aligned} 
'''
