# Copyright (c) 2013, University of Bristol and University of Southampton.

from EStat.Templating import Template

class NLevelUnorderedLogitAML(Template):
    'Fits unordered multinomial response models, in aML only, with one or more higher classifications (nested or crossed), with a logit link.'

    __version__ = '1.0.0'

    tags = [ 'Model', 'Unordered multinomial', 'N-Level' ]
    engines = ['aML']

    inputs = '''
import numpy

NumLevs = Integer('Number of Classifications: ')
for i in range(0, int(NumLevs)):
    selstr = 'Classification ' + str(i + 1) + ': '
    context['C' + str(i + 1)] = IDVector(selstr)
    context['sigmau' + str(i + 1)] = ParamScalar()

ID = context['C'+str(int(NumLevs))]

y = DataVector('Response: ')
numcats = Integer('Number of categories: ')
base = Integer('Base category: ')

lenbeta = 0
for i in range(0, int(numcats)-1):
    context['x' + str(i)] = DataMatrix('Explanatory variables for category ' + str(i + 1) + ': ', allow_cat = True)
    lenbeta += len(context['x' + str(i)])
beta = ParamVector()
beta.ncols = lenbeta
'''
    
    amlscript = '''
<%
import numpy
cats = list(set(dsets['datafile'].variables[y]['data']))
cats.remove(int(base))
%>

option maximum model space = 10000;

% for i in range(0, len(cats)):
define regset Cat${int(cats[i])}; 
var = \\
% for p in context['x' + str(i)]:
${p} \\
% endfor
;
% endfor

% for j in range(0, int(NumLevs)):
define normal distribution; dim=${len(cats)};
% for i in range(0, len(cats)):
  name=u${int(cats[i])}_${j};
% endfor
% endfor

multinomial logit model;
outcome = ${y};
% for i in range(0, len(cats)):
model ${int(cats[i])} = regset Cat${int(cats[i])}
% for j in range(0, int(NumLevs)):
        + intres(draw=${context['C' + str(j + 1)]}, ref=u${int(cats[i])}_${j})
% endfor
;
% endfor

starting values;
% for i in range(0, len(cats)):
% for j in range(0, len(context['x' + str(i)])):
beta${j}_${cats[i]}	T	0
% endfor
% endfor
% for j in range(0, int(NumLevs)):
% for i in range(0, len(cats)):
Sig${j+1}U${int(cats[i])}	T	1
% endfor
% for i in range(0, len(cats)):
% for k in range(0, i+1):
% if i != k:
Rho${j+1}U${int(cats[i])}U${int(cats[k])}	T	0
% endif
% endfor
% endfor
% endfor
;
'''
