# Copyright (c) 2013, University of Bristol and University of Southampton.

from EStat.Templating import Template

class TemplateOrderedProbitAML(Template):
    'Fits ordered multinomial response models, with a probit link; uses aML only.'

    __version__ = '1.0.0'

    tags = [ 'Model', 'Ordered multinomial', 'Probit', '1-Level' ]
    engines = ['aML']

    inputs = '''
y = DataVector('Response: ')
x = DataMatrix('Explanatory variables: ')
beta = ParamVector(parents=[x], as_scalar=True)
'''
    
    amlscript = '''
<%
import numpy
mincat = int(numpy.min(dsets['datafile'].variables[y]['data']))
maxcat = int(numpy.max(dsets['datafile'].variables[y]['data']))
numcat = maxcat - mincat
%>

define vector Taus; dim=${numcat};

define regressor set BetaX;
var = \\
% for p in x:
${p} \\
% endfor
;

ordered probit model;
outcomes = ${y}-${mincat} ${y}+${1-mincat};
thresholds = Taus;
model = regset BetaX;

starting values;
% for i in range(0, numcat):
Tau${i}	T	${i - (numcat/2.0)}
% endfor
% for i in range(0, len(x)):
beta${i}	T	0
% endfor
;
'''

    latex = r'''
\begin{aligned}
 \mbox{${y}}_i & \sim \mbox{N}(\mu_i, \sigma^2) \\ 
\mu_i & =
  ${mmulttex(x, r'\beta', 'i')} \\ 
%for i in range(0, len(x)):
\beta_${i} & \propto 1 \\ 
%endfor
\tau & \sim \Gamma (0.001,0.001) \\ 
\sigma^2 & = 1 / \tau
\end{aligned} 
'''
