# Copyright (c) 2013, University of Bristol and University of Southampton.

from EStat.Templating import Template

class Regression1AML(Template):
    'Fits 1 level Normal multiple regression models in <a href="www.applied-ml.com" target="_blank">aML</a> only.'

    __version__ = '1.0.0'

    tags = [ 'Model', '1-Level', 'Normal' ]
    engines = ['aML']

    inputs = '''
y = DataVector('Response: ', help= "<p style='text-align:left'>A.k.a. <em>Y</em>, Outcome variable, Dependent variable, etc.</p><p style='text-align:left'><strong>Note:</strong> this template assumes response is continuous.</p>")
sigma = ParamScalar()
x = DataMatrix('Explanatory variables: ', allow_cat = True, help= "<p style='text-align:left'>A.k.a. X, Predictor variables, Independent variables, etc.</p><p style='text-align:left'><strong>Note:</strong> if you wish to include an <strong>intercept</strong> then you need to add it (e.g. a constant of ones) as one of the explanatory variables.</p><p style='text-align:left'>Once you've selected a variable, you have the opportunity to indicate whether it's categorical or not; if categorical, dummy variables will be added to the model on your behalf.</p>")
beta = ParamVector(parents=[x], as_scalar=True)
'''
    
    amlscript = '''
define regressor set BetaX;
var = \\
% for p in x:
${p} \\
% endfor
;

define normal distribution; dim=1;
    name=e;

continuous model;
    outcome = ${y};
    model = regset BetaX +
        res(draw=_iid, ref=e);

starting values;
% for i in range(0, len(x)):
beta${i}	T	0
% endfor
sigma	T	1
;
'''

    latex = r'''
\begin{aligned}
 \mbox{${y}}_i & \sim \mbox{N}(\mu_i, \sigma^2) \\ 
\mu_i & =
  ${mmulttex(x, r'\beta', 'i')} \\ 
%for i in range(0, len(x)):
\beta_${i} & \propto 1 \\ 
%endfor
\tau & \sim \Gamma (0.001,0.001) \\ 
\sigma^2 & = 1 / \tau
\end{aligned} 
'''
