# Copyright (c) 2013, University of Bristol and University of Southampton.

from EStat.Templating import Template

class UnorderedLogitAML(Template):
    'Fits unordered multinomial response models, with a logit link; uses aML only.'

    __version__ = '1.0.0'

    tags = [ 'Model', 'Unordered multinomial', '1-Level', 'Logit' ]
    engines = ['aML']

    inputs = '''
import numpy
y = DataVector('Response: ')
numcats = Integer('Number of categories: ')
base = Integer('Base category: ')

lenbeta = 0
for i in range(0, int(numcats)-1):
    context['x' + str(i)] = DataMatrix('Explanatory variables for category ' + str(i + 1) + ': ')
    lenbeta += len(context['x' + str(i)])
beta = ParamVector()
beta.ncols = lenbeta
'''
    
    amlscript = '''
<%
import numpy
cats = list(set(dsets['datafile'].variables[y]['data']))
cats.remove(int(base))
%>

% for i in range(0, len(cats)):
define regset Cat${int(cats[i])}; 
var = \\
% for p in context['x' + str(i)]:
${p} \\
% endfor
;
% endfor

multinomial logit model;
outcome = ${y};
% for i in range(0, len(cats)):
model ${int(cats[i])} = regset Cat${int(cats[i])};
% endfor

starting values;
% for i in range(0, len(cats)):
% for j in range(0, len(context['x' + str(i)])):
beta${j}_${cats[i]}	T	0
% endfor
% endfor
;
'''
